# -*- coding: utf-8 -*-
#
# This file is part of s4d.
#
# s4d is a python package for speaker diarization.
# Home page: http://www-lium.univ-lemans.fr/s4d/
#
# s4d is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 3 of the License,
# or (at your option) any later version.
#
# s4d is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with s4d.  If not, see <http://www.gnu.org/licenses/>.


"""
Copyright 2014-2021 Sylvain Meignier and Anthony Larcher
"""

import copy
import numpy as np

from sidekit import Mixture, StatServer, FactorAnalyser, Scores, Ndx, PLDA_scoring, cosine_scoring, mahalanobis_scoring, two_covariance_scoring
from sidekit.sidekit_io import *


class ModelIV:
    def __init__(self, model_filename=None, nb_thread=1):
        """

        :param model_filename:
        :param nb_thread:
        """
        self.ubm = None
        self.tv = None
        self.tv_mean = None
        self.tv_sigma = None
        self.sn_mean = None
        self.sn_cov = None
        self.plda_mean = None
        self.plda_f = None
        self.plda_g = None
        self.plda_sigma = None
        self.ivectors = None
        self.scores = None

        self.nb_thread = nb_thread
        self.model_filename = model_filename
        if model_filename is not None:
            self._load_model()

    def _load_model(self):
        """

        :return:
        """
        print('load: ', self.model_filename)
        self.ubm = Mixture()
        self.ubm.read(self.model_filename, prefix='ubm/')
        self.tv, self.tv_mean, self.tv_sigma = read_tv_hdf5(self.model_filename)
        self.norm_mean, self.norm_cov = read_norm_hdf5(self.model_filename)
        self.plda_mean, self.plda_f, self.plda_g, self.plda_sigma = read_plda_hdf5(self.model_filename)

    def debug_model(self):
        """

        :return:
        """
        print('ubm_mu: ', self.ubm.mu.shape)
        print('tv: ', self.tv.shape)
        print('tv mean: ', self.tv_mean.shape)
        print('tv Sigma: ', self.tv_sigma.shape)
        if self.plda_mean is not None:
            print('plda mean: ', self.plda_mean.shape)
            print('plda Sigma: ', self.plda_sigma.shape)
            print('plda F: ', self.plda_f.shape)
            print('plda G: ', self.plda_g.shape)
        if self.sn_mean is not None:
            print('sn_mean: ', self.sn_mean.shape)
            print('sn_cov: ', self.sn_cov.shape)

    def train(self, feature_server, idmap, normalization=True):
        """

        :param feature_server:
        :param idmap:
        :param normalization:
        :return:
        """
        stat = StatServer(idmap, distrib_nb=self.ubm.distrib_nb(), feature_size=self.ubm.dim()) 
        stat.accumulate_stat(ubm=self.ubm, feature_server=feature_server, seg_indices=range(stat.segset.shape[0]), num_thread=self.nb_thread)
        stat = stat.sum_stat_per_model()[0]
        
        fa = FactorAnalyser(mean=self.tv_mean, Sigma=self.tv_sigma, F=self.tv)
        self.ivectors = fa.extract_ivectors_single(self.ubm, stat)
        
        if normalization:
            self.ivectors.spectral_norm_stat1(self.norm_mean[:1], self.norm_cov[:1])

        return self.ivectors

    def train_per_segment(self, feature_server, idmap, normalization=True):
        """

        :param feature_server:
        :param idmap:
        :param normalization:
        :return:
        """
        stat = StatServer(idmap, distrib_nb=self.ubm.distrib_nb(), feature_size=self.ubm.dim())
        stat.accumulate_stat(ubm=self.ubm, feature_server=feature_server, seg_indices=range(stat.segset.shape[0]),
                             num_thread=self.nb_thread)
        fa = FactorAnalyser(mean=self.tv_mean, Sigma=self.tv_sigma, F=self.tv)
        ivectors = fa.extract_ivectors_single(self.ubm, stat)

        if normalization:
            ivectors.spectral_norm_stat1(self.norm_mean[:1], self.norm_cov[:1])

        return ivectors

    def score_cosine(self, use_wccn=True):
        """

        :param use_wccn:
        :return:
        """
        wccn = None
        if use_wccn:
            wccn = read_key_hdf5(self.model_filename, 'wccn_choleski')
        ndx = Ndx(models=self.ivectors.modelset, testsegs=self.ivectors.modelset)
        self.scores = cosine_scoring(self.ivectors, self.ivectors, ndx, wccn=wccn, check_missing=False)

        return self.scores

    def score_mahalanobis(self, use_covariance=True):
        """

        :param use_covariance:
        :return:
        """
        if use_covariance:
            m = read_key_hdf5(self.model_filename, 'mahalanobis_matrix')
        else:
            m = numpy.identity(self.tv.shape[2])
        ndx = Ndx(models=self.ivectors.modelset, testsegs=self.ivectors.modelset)
        self.scores = mahalanobis_scoring(self.ivectors, self.ivectors, ndx, m, check_missing=False)

        return self.scores

    def score_two_covariance(self):
        """

        :return:
        """
        W = read_key_hdf5(self.model_filename, 'two_covariance/within_covariance')
        B = read_key_hdf5(self.model_filename, 'two_covariance/between_covariance')
        ndx = Ndx(models=self.ivectors.modelset, testsegs=self.ivectors.modelset)
        self.scores = two_covariance_scoring(self.ivectors, self.ivectors, ndx, W, B, check_missing=False)

        return self.scores

    def score_plda(self):
        """

        :return:
        """
        self.plda_mean, self.plda_f, self.plda_g, self.plda_sigma = read_plda_hdf5(self.file_name)
        ndx = Ndx(models=self.ivectors.modelset, testsegs=self.ivectors.modelset)

        self.scores = PLDA_scoring(self.ivectors, self.ivectors, ndx, self.plda_mean, self.plda_f, self.plda_g, self.plda_sigma, p_known=0.0)

        return self.scores

#    def update(self, i, j):
#        cluster_list = self.diarization.make_index(['cluster'])
#
#        stat_server_merge(self.ivectors, i, j, 1, 1)
#        self.scores = self.score_plda()

    def score_plda_slow(self):
        """

        :return:
        """
        self.plda_mean, self.plda_f, self.plda_g, self.plda_sigma = read_plda_hdf5(self.model_filename)
        local_ndx = Ndx(models=self.ivectors.modelset, testsegs=self.ivectors.modelset)

        enroll_copy = copy.deepcopy(self.ivectors)

        # Center the i-vectors around the PLDA mean
        enroll_copy.center_stat1(self.plda_mean)

        # Compute temporary matrices
        invSigma = np.linalg.inv(self.plda_sigma)
        I_iv = np.eye(self.plda_mean.shape[0], dtype='float')
        I_ch = np.eye(self.plda_g.shape[1], dtype='float')
        I_spk = np.eye(self.plda_f.shape[1], dtype='float')
        A = np.linalg.inv(self.plda_g.T.dot(invSigma).dot(self.plda_g) + I_ch)
        B = self.plda_f.T.dot(invSigma).dot(I_iv - self.plda_g.dot(A).dot(self.plda_g.T).dot(invSigma))
        K = B.dot(self.plda_f)
        K1 = np.linalg.inv(K + I_spk)
        K2 = np.linalg.inv(2 * K + I_spk)

        # Compute the Gaussian distribution constant
        alpha1 = np.linalg.slogdet(K1)[1]
        alpha2 = np.linalg.slogdet(K2)[1]
        constant = alpha2 / 2.0 - alpha1

        # Compute verification scores
        l = enroll_copy.segset.shape[0]
        scores = Scores()
        scores.scoremat = np.zeros((l, l))
        scores.modelset = enroll_copy.modelset
        scores.segset = enroll_copy.modelset
        scores.scoremask = local_ndx.trialmask

        # Project data in the space that maximizes the speaker separability
        enroll_tmp = B.dot(enroll_copy.stat1.T)

        # Compute verification scores
        # Loop on the models
        for model_idx in range(enroll_copy.modelset.shape[0]):

            s2 = enroll_tmp[:, model_idx].dot(K1).dot(enroll_tmp[:, model_idx])

            mod_plus_test_seg = enroll_tmp + np.atleast_2d(enroll_tmp[:, model_idx]).T

            tmp1 = enroll_tmp.T.dot(K1)
            tmp2 = mod_plus_test_seg.T.dot(K2)

            for seg_idx in range(model_idx, enroll_copy.segset.shape[0]):
                s1 = tmp1[seg_idx, :].dot(enroll_tmp[:, seg_idx])
                s3 = tmp2[seg_idx, :].dot(mod_plus_test_seg[:, seg_idx])
                scores.scoremat[model_idx, seg_idx] = (s3 - s1 - s2)/2. + constant
                scores.scoremat[seg_idx, model_idx] = (s3 - s1 - s2)/2. + constant
        self.scores = scores
        return scores

#def plda_scores_from_diar(model_fn, feature_server, diar, idmap, return_model=False):
#    model_iv = ModelIV(model_filename=model_fn, feature_server=feature_server, diarization=diar, idmap=idmap)

#    model_iv.train()
#    scores = model_iv.score_plda()
#    if return_model:
#        return scores, model_iv
#    return scores

#def cosine_scores_from_diar(model_fn, feature_server, diar, return_model=False):
#    model_iv = ModelIV(model_filename=model_fn, feature_server=feature_server, diarization=diar)
    #if vad:
    #model_iv.vad()

#    model_iv.train(normalization=False)
#    scores = model_iv.score_cosine()
#    if return_model:
#        return scores, model_iv
#    return scores



# -*- coding: utf-8 -*-
#
# This file is part of s4d.
#
# s4d is a python package for speaker diarization.
# Home page: http://www-lium.univ-lemans.fr/s4d/
#
# s4d is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 3 of the License,
# or (at your option) any later version.
#
# s4d is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with s4d.  If not, see <http://www.gnu.org/licenses/>.


"""
Copyright 2014-2021 Sylvain Meignier
"""

import copy
import numpy as np
import logging


def argmin(distances, nb):
    """
    Get argmin and min indexes between 0 and nb of a distance matrix
    :param distances: a numpy.ndarray
    :param nb: int
    :return: row and column indexes, the value
    """
    if nb <= 1:
        return 0, 0, np.inf
    tmp_dist = distances[0:nb, 0:nb]
    # numpy.nanargmin : give the absolute position in the matrix, ie 1 number
    # unravel_index: give the row and col positions
    try:
        i, j = np.unravel_index(np.nanargmin(tmp_dist), tmp_dist.shape)
    except ValueError:
        logging.warning('value are NaN, nb:'+str(nb))
        logging.warning(distances)
        logging.warning(tmp_dist)
        return 0, 0, np.inf

    v = distances[i, j]
    return i, j, v

def argmax(distances, nb):
    """
    Get argmin and min indexes between 0 and nb of a distance matrix
    :param distances: a numpy.ndarray
    :param nb: int
    :return: row and column indexes, the value
    """
    if nb <= 1:
        return 0, 0, np.inf
    tmp_dist = distances[0:nb, 0:nb]
    # numpy.nanargmin : give the absolute position in the matrix, ie 1 number
    # unravel_index: give the row and col positions
    try:
        i, j = np.unravel_index(np.nanargmax(tmp_dist), tmp_dist.shape)
    except ValueError:
        logging.warning('value are NaN, nb:'+str(nb))
        logging.warning(distances)
        logging.warning(tmp_dist)
        return 0, 0, -np.inf

    v = distances[i, j]
    return i, j, v


def roll(mat, j):
    """
    delete the line j and column j in the matrix
    :param mat: numpy.ndarray
    :param j: int
    :return: numpy.ndarray
    """
    return np.delete(np.delete(mat, j, 1), j, 0)


def bic_square_root(ni, nj, alpha, dim):
    """
    Compute a BIC square root distance described in [Stafylakis2010]_.

    .. [Stafylakis2010] T. Stafylakis, V. Katsouros, and G. Carayannis. The segmental bayesian information criterion and its applications to speaker diarization. Selected Topics in Signal Processing, IEEE Journal of, 4(5):857-866, 2010.

    :param ni: covariance matrix of speaker i
    :param nj: covariance matrix of speaker j
    :param alpha: a threshold
    :param dim: the dimenssion of the features
    :return: a float
    """
    nij = ni + nj
    constant_covariance = 0.5 * alpha * (0.5 * ((dim + 1) * dim))
    constant_mean = 0.5 * alpha * dim
    mean = ((np.sqrt(ni) * np.log(ni)) + (np.sqrt(nj) * np.log(nj))) - (np.sqrt(nij) * np.log(nij))
    covariance = (np.log(ni) + np.log(nj)) - np.log(nij)
    #print(ni, nj, nij, alpha, dim, constant_covariance, constant_mean, mean, covariance)
    return (constant_covariance * covariance) + (constant_mean * mean)

def stat_server_remove(stat_server, index):
    """
    " remove data at position index
    :param index: the index to remove
    """
    stat_server.segset = np.delete(stat_server.segset, index)
    stat_server.modelset = np.delete(stat_server.modelset, index)
    stat_server.start = np.delete(stat_server.start, index)
    stat_server.stop = np.delete(stat_server.stop, index)
    stat_server.stat0 = np.delete(stat_server.stat0, index, axis=0)
    stat_server.stat1 = np.delete(stat_server.stat1, index, axis=0)


def stat_server_merge(stat_server, i, j, wi=1.0, wj=1.0):
    """
    merge the ith and jth stat0 and stat1 into ith data, remove jth data
    :param i: index destination
    :param j: index removed
    """
    if stat_server.stop[i] != 0 and stat_server.stop[i] is not None:
        logging.warning('segment information will be wrong')
    stat_server.stat0[i, :] = (wi * stat_server.stat0[j, :] + wj * stat_server.stat0[i, :]) / (wi + wj)
    stat_server.stat1[i, :] = (wi * stat_server.stat1[j, :] + wj * stat_server.stat1[i, :]) / (wi + wj)
    stat_server_remove(stat_server, j)


def idmap_remove(idmap, index):
    """
    " remove data at position index
    :param index: the index to remove
    """
    idmap.leftids = np.delete(idmap.leftids, index)
    idmap.rightids = np.delete(idmap.rightids, index)
    idmap.start = np.delete(idmap.start, index)
    idmap.stop = np.delete(idmap.stop, index)


def scores_remove(scores, index_model=None, index_seg=None):
    """
    " remove data at position index_model and/or index_seg
    :param index_model: the index in model set to remove
    :param index_seg: the index in segment set to remove
    """

    if index_seg is not None:
        scores.segset = np.delete(scores.segset, index_seg)
        scores.scoremask = np.delete(scores.scoremask, index_seg, axis=1)
        scores.scoremat = np.delete(scores.scoremat, index_seg, axis=1)

    if index_model is not None:
        scores.modelset = np.delete(scores.modelset, index_model)
        scores.scoremask = np.delete(scores.scoremask, index_model, axis=0)
        scores.scoremat = np.delete(scores.scoremat, index_model, axis=0)


def scores2distance(scores, threshold):
    distance = (scores.scoremat + scores.scoremat.T) / 2.0 * -1.0
    np.fill_diagonal(distance, np.inf)
    min = np.min(distance)-1
    distance -= min
    np.fill_diagonal(distance, 0.0)
    t = -1.0 * threshold - min
    return distance, t




# -*- coding: utf-8 -*-
#
# This file is part of s4d.
#
# s4d is a python package for speaker diarization.
# Home page: http://www-lium.univ-lemans.fr/s4d/
#
# s4d is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 3 of the License,
# or (at your option) any later version.
#
# s4d is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with s4d.  If not, see <http://www.gnu.org/licenses/>.


"""
Copyright 2014-2021 Sylvain Meignier
"""



# -*- coding: utf-8 -*-
#
# This file is part of s4d.
#
# s4d is a python package for speaker diarization.
# Home page: http://www-lium.univ-lemans.fr/s4d/
#
# s4d is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 3 of the License,
# or (at your option) any later version.
#
# s4d is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with s4d.  If not, see <http://www.gnu.org/licenses/>.


"""
Copyright 2014-2021 Sylvain Meignier
"""

import copy
import logging
import numpy as np

from ..diar import Diar
from .gauss import GaussFull
from .hac_utils import argmin, roll
from .hac_utils import bic_square_root
from math import isnan


class HAC_BIC:
    """
    BIC Hierarchical Agglomerative Clustering (HAC) with gaussian models

    The algorithm is based upon a hierarchical agglomerative clustering. The
    initial set of clusters is composed of one segment per cluster. Each cluster
    is modeled by a Gaussian with a full covariance matrix (see
    :class:`gauss.GaussFull`). :math:`\Delta BIC`
    measure is employed to select the candidate clusters to group as well as
    to stop the merging process. The two closest clusters :math:`i` and
    :math:`j` are merged at each iteration until :math:`\\Delta BIC_{i,j} > 0`.


        :math:`\\Delta BIC_{i,j} = PBIC_{i+j} - PBIC_{i} - PBIC_{j} - P`

        :math:`PBIC_{x}  = \\frac{n_x}{2} \\log|\\Sigma_x|`

        :math:`cst  = \\frac{1}{2} \\alpha \\left(d + \\frac{d(d+1)}{2}\\right)`

        :math:`P  = cst + log(n_i+n_j)`

    where :math:`|\\Sigma_i|`, :math:`|\\Sigma_j|` and :math:`|\\Sigma|` are the
    determinants of gaussians associated to the clusters :math:`i`, :math:`j`
    and :math:`i+j`. :math:`\\alpha` is a parameter to set up. The penalty
    factor :math:`P` depends on :math:`d`, the dimension of the features, as well as
    on :math:`n_i` and :math:`n_j`, refering to the total length of cluster
    :math:`i` and cluster :math:`j` respectively.


    """
    def __init__(self, cep, table, alpha=1.0, sr=False):
        self.cep = cep
        self.dim = cep.shape[1]
        self.alpha = alpha
        self.diar = copy.deepcopy(table)
        self.models = []
        self.merge = []
        self.nb_merge = 0
        self.sr = sr
        self.dist = None
        self._init_train()
        self._init_distance()

    def _init_train(self):
        """
        Train initial models

        """
        map = self.diar.make_index(['cluster'])
        for cluster in map:
            model = GaussFull(cluster, self.dim)
            self.models.append(model)
            self.cst_bic = GaussFull.cst_bic(self.dim, self.alpha)
            for row in map[cluster]:
                start = row['start']
                stop = row['stop']
                model.add(self.cep[start:stop])

        for model in self.models:
            model.compute()

    def _init_distance(self):
        """ Compute distance matrix
        """
        nb = len(self.models)
        self.dist = np.full((nb, nb), np.nan)
        # for i in range(0, nb):
        #    mi = self.models[i]
        for i, mi in enumerate(self.models):
            # self.merge.append([])
            # for j, mj in enumerate(self.models, start=i+1):
            #    logging.debug('i %d j %d n %d', i, j ,nb)
            for j in range(i + 1, nb):
                mj = self.models[j]
                self.dist[i, j] = self.dist[j, i] = self._dist(mi, mj)
        # logging.debug(self.dist)

    def _dist(self, mi, mj):
        """
        Compute the BIC distance d(i,j)
        :param mi: a GaussFull object
        :param mj: a GaussFull object
        :return: float
        """
        v = GaussFull.merge_partial_bic(mi, mj) - mi.partial_bic - mj.partial_bic
        if self.sr:
            v += - bic_square_root(mi.count, mj.count, self.alpha, self.dim)
        else:
            v += - self.cst_bic * np.log(mi.count + mj.count)
        if isnan(v):
            logging.warning('BIC is NAN, mi: '+mi.name+' ' + str(mi.count)+' mj: '+mj.name+' ' + str(mj.count))
        return v

    def _merge_model(self, mi, mj):
        """
        Merge two a GaussFull objects
        :param mi: a GaussFull object
        :param mj: a GaussFull object
        :return: a GaussFull object
        """
        return GaussFull.merge(mi, mj)

    def _update_dist(self, i):
        """
        Update row and column i of the distance matrix
        :param i: int

        """
        nb = len(self.models)
        mi = self.models[i]
        for j in (x for x in range(nb) if x != i):
            mj = self.models[j]
            self.dist[i, j] = self.dist[j, i] = self._dist(mi, mj)

    def information(self, i, j, value, duration):
        self.merge.append([self.nb_merge, self.models[i].name, self.models[j].name, value, duration])

    def perform(self, to_the_end=False):
        """
        perform the HAC algorithm
        :return: a Diar object and a dictonary mapping the old cluster_list to the
        new lables
        """
        nb = len(self.models)
        self.nb_merge = -1
        for i in range(nb):
            self.information(i, i, 0, self.models[i].count)

        i, j, v = argmin(self.dist, nb)
        self.nb_merge = 0
        while v < 0.0 and nb > 1:
            self.information(i, j, v, self.models[i].count+self.models[j].count)
            self.nb_merge += 1
            logging.debug('merge: %d c1: %s (%d) c2: %s (%d) dist: %f %d',
                          self.nb_merge, self.models[i].name, i,
                          self.models[j].name, j, v, nb)
            # update merge
            # self.merge[i].append(
            #    [self.nb_merge, self.models[i].speaker, self.models[j].speaker, v])
            # self.merge[i] += self.merge[j]
            # self.merge.pop(j)
            self.diar.rename('cluster', [self.models[j].name], self.models[i].name)
            # update model
            self.models[i] = self._merge_model(self.models[i], self.models[j])
            self.models.pop(j)
            # nb = len(self.models)
            # update distances
            self.dist = roll(self.dist, j)
            self._update_dist(i)
            nb -= 1
            i, j, v = argmin(self.dist, nb)

        out_diar = copy.deepcopy(self.diar)

        if to_the_end:
            while nb > 1:
                self.information(i, j, v, self.models[i].count+self.models[j].count)
                self.nb_merge += 1
                logging.debug('merge: %d c1: %s (%d) c2: %s (%d) dist: %f %d',
                              self.nb_merge, self.models[i].name, i,
                              self.models[j].name, j, v, nb)
                self.diar.rename('cluster', [self.models[j].name], self.models[i].name)
                # update model
                self.models[i] = self._merge_model(self.models[i], self.models[j])
                self.models.pop(j)
                # nb = len(self.models)
                # update distances
                self.dist = roll(self.dist, j)
                self._update_dist(i)
                nb -= 1
                i, j, v = argmin(self.dist, nb)

        return out_diar


def hac_bic(feature_server, diar, threshold, square_root_bic = False):
    """

    :param feature_server:
    :param diar:
    :param threshold:
    :param square_root_bic:
    :return:
    """
    shows = diar.make_index(['show'])
    diar_out = Diar()
    for show in shows:
        cep, _ = feature_server.load(show)
        bic = HAC_BIC(cep, shows[show], alpha=threshold, sr=square_root_bic)
        diar_out += bic.perform(to_the_end=True)
    return diar_out



# -*- coding: utf-8 -*-
#
# This file is part of s4d.
#
# s4d is a python package for speaker diarization.
# Home page: http://www-lium.univ-lemans.fr/s4d/
#
# s4d is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 3 of the License,
# or (at your option) any later version.
#
# s4d is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with s4d.  If not, see <http://www.gnu.org/licenses/>.


"""
Copyright 2014-2021 Sylvain Meignier and Anthony Larcher
"""

from .clustering.hac_bic import HAC_BIC
from .clustering.hac_clr import HAC_CLR
from  .clustering.hac_iv import hac_iv, hac_iv_it

from .clustering.hac_utils import argmin
from .clustering.hac_utils import argmax
from .clustering.hac_utils import roll
from .clustering.hac_utils import stat_server_remove
from .clustering.hac_utils import scores_remove
from .clustering.hac_utils import scores2distance
from .clustering.hac_utils import stat_server_merge
from .clustering.hac_utils import bic_square_root

from .clustering.cc_iv import ConnectedComponent

from .nnet.wavsets import SeqSet
from .nnet.seqtoseq import BLSTM

from .model_iv import ModelIV

from .diar import Diar
from .segmentation import sanity_check
from .segmentation import bic_linear
from .segmentation import div_gauss

from .viterbi import Viterbi

from .scoring import DER


__author__ = "Sylvain Meignier and Anthony Larcher"
__copyright__ = "Copyright 2014-2021 Sylvain Meignier and Anthony Larcher"
__license__ = "LGPL"
__maintainer__ = "Sylvain Meignier"
__email__ = "sylvain.meignierr@univ-lemans.fr"
__status__ = "Production"
__docformat__ = 'reStructuredText'
__version__ = "0.1.6.3"
